#
# Copyright 2006 sense.lab e.V.
# 
# This file is part of the CryptoBox.
#
# The CryptoBox is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# The CryptoBox is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the CryptoBox; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""The network feature of the CryptoBox.
"""

__revision__ = "$Id"

import subprocess
import os
import cryptobox.plugins.base


## specify (in seconds), how long we should wait before redirecting and ip change
REDIRECT_DELAY = 10
CHANGE_IP_DELAY = 2 

class network(cryptobox.plugins.base.CryptoBoxPlugin):
	"""The network feature of the CryptoBox.
	"""

	plugin_capabilities = [ "system" ]
	plugin_visibility = [ "preferences" ]
	request_auth = True
	rank = 30

	def do_action(self, store=None, redirected="", ip1="", ip2="", ip3="", ip4=""):
		"""Show a form containing the current IP - change it if requested.
		"""
		## if we were redirected, then we should display the default page
		self.cbox.log.debug("executing network plugin")
		if redirected == "1":
			self.cbox.log.debug("network plugin: redirected")
			return "form_network"
		## request for IP change?
		if store:
			self.cbox.log.debug("network plugin: changing IP")
			try:
				for ip_in in (ip1, ip2, ip3, ip4):
					if (int(ip_in) < 0) or (int(ip_in) > 255):
						self.cbox.log.info("invalid IP supplied: %s" % \
								str((ip1, ip2, ip3, ip4)))
						raise ValueError
				new_ip = "%d.%d.%d.%d" % (int(ip1), int(ip2), int(ip3), int(ip4))
			except ValueError:
				self.hdf["Data.Warning"] = "Plugins.network.InvalidIP"
				self.__prepare_form_data()
				return "form_network"
			if self.__set_ip(new_ip):
				self.cbox.log.info("the IP was successfully changed: %s" % new_ip)
				self.hdf["Data.Success"] = "Plugins.network.IPChanged"
				self.hdf["Data.Redirect.URL"] = self.__get_redirect_destination(new_ip)
				self.hdf["Data.Redirect.Delay"] = REDIRECT_DELAY
				return "form_network"
			else:
				self.cbox.log.warn("failed to change IP address to: %s" % new_ip)
				self.hdf["Data.Warning"] = "Plugins.network.InvalidIP"
				self.__prepare_form_data()
				return "form_network"
		else:
			self.cbox.log.debug("network plugin: show form")
			## just show the form
			self.__prepare_form_data()
			return "form_network"


	def get_status(self):
		"""The current IP is the status of this feature.
		"""
		return "%d.%d.%d.%d" % self.__get_current_ip()


	def __get_redirect_destination(self, ip):
		"""Put the new URL together.
		"""
		import cherrypy
		req = cherrypy.request
		base_parts = req.base.split(":")
		dest = "%s://%s" % (base_parts[0], ip)
		if len(base_parts) == 3:
			dest += ":%s" % base_parts[2]
		return dest


	def __prepare_form_data(self):
		"""Set some hdf values.
		"""
		(oc1, oc2, oc3, oc4) = self.__get_current_ip()
		self.hdf[self.hdf_prefix + "ip.oc1"] = oc1
		self.hdf[self.hdf_prefix + "ip.oc2"] = oc2
		self.hdf[self.hdf_prefix + "ip.oc3"] = oc3
		self.hdf[self.hdf_prefix + "ip.oc4"] = oc4


	def __get_current_ip(self):
		"""Retrieve the current IP.
		"""
		import re
		import imp
		## load some values from the root_action.py script
		root_action_plug = imp.load_source("root_action",
				os.path.join(self.plugin_dir, "root_action.py"))
		## get the current IP of the network interface
		proc = subprocess.Popen(
			shell = False,
			stdout = subprocess.PIPE,
			args = [
				root_action_plug.IFCONFIG_BIN,
				root_action_plug.IFACE])
		(stdout, stderr) = proc.communicate()
		if proc.returncode != 0:
			return (0,0,0,0)
		## this regex matches the four numbers of the IP
		match = re.search(r'inet [\w]+:(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})\s', stdout)
		if match:
			## use the previously matched numbers
			return tuple([int(e) for e in match.groups()])
		else:
			return (0, 0, 0, 0)


	def __set_ip(self, new_ip):
		"""Change the IP.
		"""
		import threading
		## call the root_action script after some seconds - so we can deliver the page before
		def delayed_ip_change():
			"""A threaded function to change the IP.
			"""
			import time
			time.sleep(CHANGE_IP_DELAY)
			proc = subprocess.Popen(
				shell = False,
				stderr = subprocess.PIPE,
				args = [
					self.cbox.prefs["Programs"]["super"],
					self.cbox.prefs["Programs"]["CryptoBoxRootActions"],
					"plugin",
					os.path.join(self.plugin_dir, "root_action.py"),
					new_ip])
			proc.wait()
			if proc.returncode != 0:
				self.cbox.log.warn("failed to change IP address: %s" % new_ip)
				self.cbox.log.warn("error output: %s" % str(proc.stderr.read()))
			return
		thread = threading.Thread()
		thread.run = delayed_ip_change
		thread.setDaemon(True)
		thread.start()
		# TODO: how could we guess, if it failed?
		return True

