#
# Copyright 2006 sense.lab e.V.
# 
# This file is part of the CryptoBox.
#
# The CryptoBox is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# The CryptoBox is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the CryptoBox; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__revision__ = "$Id"

import cryptobox.plugins.base
import cryptobox.core.container
from cryptobox.core.exceptions import *


class volume_automount(cryptobox.plugins.base.CryptoBoxPlugin):

	plugin_capabilities = [ "volume" ]
	plugin_visibility = [ "properties" ]
	request_auth = False
	rank = 80

	true_string = "yes"
	false_string = "no"


	def do_action(self, action=None):
		container = self.cbox.get_container(self.device)
		if action is None:
			pass
		elif action == "enable":
			container.attributes["automount"] = self.true_string
			self.hdf["Data.Success"] = "Plugins.volume_automount.AutoMountEnabled"
			self.cbox.log.info("volume_automount: enabled for device '%s'" % self.device)
			try:
				self.cbox.prefs.volumes_db.write()
			except IOError:
				self.cbox.log.warn("Failed to store volume database")
		elif action == "disable":
			container.attributes["automount"] = self.false_string
			self.hdf["Data.Success"] = "Plugins.volume_automount.AutoMountDisabled"
			self.cbox.log.info("volume_automount: disabled for device '%s'" % self.device)
			try:
				self.cbox.prefs.volumes_db.write()
			except IOError:
				self.cbox.log.warn("Failed to store volume database")
		else:
			self.cbox.log.info("volume_automount: invalid action (%s)" % str(action))
		self.__prepare_hdf()
		return "volume_automount"
	

	def setup(self):
		"""Override bootup behaviour.
		
		Mount all volumes marked as 'automount'.
		"""
		cryptobox.plugins.base.CryptoBoxPlugin.setup(self)
		for cont in self.cbox.get_container_list():
			if self.__is_auto_mount(cont) and not cont.is_mounted():
				cont.mount()
	

	def is_useful(self, device):
		"""Automount does not work for encrypted volumes.
		"""
		from cryptobox.core.container import CONTAINERTYPES as cont_types
		cont = self.cbox.get_container(device)
		if not cont:
			return False
		if cont.get_type() != cont_types["luks"]:
			return True
		return False


	def get_status(self):
		return str(self.__is_auto_mount(self.cbox.get_container(self.device)))
	

	def __prepare_hdf(self):
		if self.__is_auto_mount(self.cbox.get_container(self.device)):
			self.hdf[self.hdf_prefix + "automount_setting"] = "1"
		else:
			self.hdf[self.hdf_prefix + "automount_setting"] = "0"


	def __is_auto_mount(self, container):
		if not container:
			return False
		## only valid for plain volumes
		if container.get_type() != cryptobox.core.container.CONTAINERTYPES["plain"]:
			return False
		if container.attributes.has_key("automount"):
			return container.attributes["automount"] == self.true_string
		else:
			return False

